import java.util.*;
import java.io.StreamTokenizer;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileReader;
import java.io.IOException;

//********************************************************************
//  Final Project.java Author: Hahna Kane and Rio Akasaka
//  
//  Last modified: Friday, May 11, 2007
//
//  Description: Reads airplane.txt and allows user to find direct
//  flights, direct flights from one city to another, add flights,
//  add new airports, and display all flights to a particular city.
//  
//********************************************************************

public class FinalProject extends Graph
{
  public static void main (String [] args) throws IOException
  {
    StreamTokenizer kbdin = new StreamTokenizer 
      (new InputStreamReader(System.in));
    String response, name;
    boolean available; // Determiner for available entries
    Graph cities = new Graph();
    Timer time = new Timer();
    Map<String, Airport> airports = new HashMap<String, Airport>( );
    
    try
      {
	FileReader fin = new FileReader( args[0] );
	BufferedReader graphFile = new BufferedReader( fin );
	String line;
	int count = 0;
	
	//------------------------------------------------------------------
	// Read file, add edges and vertices
	//------------------------------------------------------------------
	
	String acronym = "";
	String val;
	String store = "";
	String flight = "";
	int flightnum, departime;
	int timedelay, ycord, xcord;
	
	// Read airport information
	while( ( line = graphFile.readLine( ) ) != null )
	  {
	    if (line.length() == 0) continue;
	    StringTokenizer st = new StringTokenizer( line );
	    
	    // Ignore lines that begin with #
	    while ((val = st.nextToken( )).startsWith("#")) {
	      line = graphFile.readLine( );
	      st = new StringTokenizer( line ); }
	    
	    if (val.startsWith("!")) break;
	    else {		
	      String wholename ="";
	      acronym = val;
	      timedelay = Integer.parseInt( st.nextToken( ).substring(0,3) );
	      xcord = Integer.parseInt( st.nextToken( ) );
	      ycord = Integer.parseInt( st.nextToken( ) );
	      while (st.hasMoreTokens( )) {
		wholename = (wholename + (store = st.nextToken( )) + " ");
	      }
	      // Stores into airport object
	      Airport airp = new Airport(acronym, timedelay, 
					 xcord, ycord, wholename);
	      cities.addvertex(new Vertex(airp));
	      airports.put(airp.getAcronym(), airp);
	    }
	  }
	
	// Read route information
	while( ( line = graphFile.readLine( ) ) != null && count != 31) {
	  count++;
	  StringTokenizer st = new StringTokenizer( line ); 
	  String from = st.nextToken(); // Store this into vertex as from city
	  String to = st.nextToken(); // Store this into vertex as to city
	}
	
	// Read general flight data	
	while( ( line = graphFile.readLine( ) ) != null )
	  {
	    StringTokenizer st = new StringTokenizer( line );
	    if (line.length() == 0) continue;
	    
	    // Ignore lines that begin with #
	    while ((val = st.nextToken( )).startsWith("#")) {
	      line = graphFile.readLine( );
	      st = new StringTokenizer( line ); }
	    
	    // If both flight and hours are concatenated
	    if (val.length( ) > 2)  {
	      flight = val; // Temporary storage
	      val = val.substring(0,2);
	      flightnum = Integer.parseInt(flight.substring(2,6));
	    }
	    else {
	      flightnum = Integer.parseInt(st.nextToken());
	    }
	    
	    String departs = st.nextToken();
	    FlightInfo plane = new FlightInfo(val, flightnum, 0);
	    
	    // Deals with departure time, sends departure city (for
	    // time adjustment) and 0 for departure, 1 for arrival (it
	    // calculates time adjustment only if 1)
	    time.manageTime(st, 0, plane);
	    String arrives = st.nextToken();
	    
	    time.manageTime(st, 1, plane);
	    
	    int stops = 0;
	    try
	      { stops = Integer.parseInt(st.nextToken( )); }
	    catch (NumberFormatException e)
	      { stops = Integer.parseInt(st.nextToken( )); }
	    plane.addNumStops(stops);
	    
	    /*Add edges*/
	    Vertex vfrom = null, vto = null, temp;
	    count = 0;

	    vfrom = cities.nodes.get(departs); 
	    vto = cities.nodes.get(arrives); 
	    
	    if (vfrom==null)
	      {
		System.out.println("Could not find vertex equals "+departs);
		break;
	      }
	    if (vto==null)
	      {
		System.out.println("Could not find vertex equals "+arrives);
		break;
	      }

	    // Calculate flight time
	    int depart = plane.getDepart(); // Returns departure time
	    String depA = plane.getDampm(); // Returns arrival am or pm
	    int arrive = plane.getArrival(); // Returns arrival time
	    String arriveA = plane.getAampm(); // Returns arrival time
	    manageAmPm(depart, depA, arrive, 
		       arriveA, vfrom, vto,
		       plane, cities); 	
	    
	  }
      }
    catch( IOException e )
      { System.err.println( e ); }
    System.out.println( "File successfully read." );
    
    menu();
    kbdin.nextToken();
    response = kbdin.sval;
    while ( !response.equals("q") ) 
      {
	switch( response.charAt(0) ) 
	  {
	    
	    //----------------------------------------------------------------
	    // Insert a new city
	    //----------------------------------------------------------------
	  case 'i': 
	    System.out.println("Insert a new city ");
	    System.out.println("Please enter the acronym: ");
	    System.out.flush(); kbdin.nextToken(); 
	    String acronym = kbdin.sval;
	    System.out.println("Please enter the name of the city: ");
	    System.out.flush(); kbdin.nextToken();
	    String wholename = kbdin.sval;
	    System.out.println("Please enter the x coordinates: ");
	    System.out.flush(); kbdin.nextToken(); 
	    int xcord = (int) kbdin.nval;
	    System.out.println("Please enter the y coordinates: ");
	    System.out.flush(); kbdin.nextToken(); 
	    int ycord = (int) kbdin.nval;
	    System.out.println("Please enter the GMT offset : ");
	    System.out.flush(); kbdin.nextToken(); 
	    int timedelay = (int) kbdin.nval;
	    Airport airp = new Airport(acronym, timedelay, xcord,
				       ycord, wholename);
	    cities.addvertex(new Vertex(airp));
	    airports.put(airp.getAcronym(), airp);
	    break;
	    
    
	    //----------------------------------------------------------------
	    // Display direct flights going from A to B
	    //----------------------------------------------------------------
	  case 'd':
	    System.out.println("Please enter the originating city: ");
	    System.out.flush(); kbdin.nextToken(); 
	    String fromCity = kbdin.sval;
	    if (!airports.containsKey(fromCity)) {
	    System.out.println("Please enter valid city."); continue;
	    }
	    System.out.println("Please enter the destination city: ");
	    System.out.flush(); kbdin.nextToken();
	    String toCity = kbdin.sval;
	    if (!airports.containsKey(toCity)) {
	      System.out.println("Please enter valid city."); continue;
	    }
	    Vertex toCityVert = new Vertex(airports.get(toCity));
	    boolean found = false;
	    int count = 0;
	    
	    Vertex temp = null;
	    if ((temp = cities.nodes.get(fromCity)) != null)
	    { 
	   	    ListIterator tovert = temp.adj.listIterator(0);
		    while (tovert.hasNext())
		      {
			Edge directEdge = (Edge) tovert.next();
			Vertex direct = directEdge.getInfo().getDest();
			if (direct.equals(toCityVert))
			  {
			    found = true; // If it finds a flight to dest
			    if (directEdge.getInfo().getNumStops() == 0)
			      {
				System.out.println(directEdge.getInfo());
			    time.printTotal(directEdge.getCost(), 2);
				count = 1; // If it finds a direct flight
			      }
			  }
		      }
	    }
	    if (!found || count == 0)
	      System.out.println("There are no direct flights from " 
				 + fromCity + " to " + toCity + ".");
	    break;
	    
	    
	    //----------------------------------------------------------------
	    // Display flights coming from A
	    //----------------------------------------------------------------
	  case 'f':
	    available = false;
	    System.out.println("Please enter originating city: ");
	    System.out.flush(); kbdin.nextToken( );
	    String origCity = kbdin.sval;
	    if (!airports.containsKey(origCity)) {
	      System.out.println("Please enter valid city."); continue;
	    }
	    System.out.println("\nFlights departing from: " 
			       + (airports.get(origCity)).getName());
	    if ((temp = cities.nodes.get(origCity)) != null)
	      {
		ListIterator tovert = temp.adj.listIterator(0);
		while (tovert.hasNext( )) {
		  available = true;
		  Edge edgeFlight = (Edge) tovert.next( );
		  FlightInfo flights = edgeFlight.getInfo( );
		  System.out.println( flights );
		}
	      }
	    if (!available) System.out.println("There are no flights from " 
					       + origCity + ".");
	    break;
	    
	    //----------------------------------------------------------------
	    // Display flights going to city B
	    //----------------------------------------------------------------
	  case 'r':
	    System.out.println("Please enter destination city: ");
	    System.out.flush(); kbdin.nextToken();
	    String destCity = kbdin.sval;
	    if (!airports.containsKey(destCity)) {
	      System.out.println("Please enter valid city."); continue;
	    }
	    System.out.println("\nFlights arriving at: " 
	    + (airports.get(destCity)).getName());
	    Vertex destvert = new Vertex(airports.get(destCity));
	    available = false;
	    
	    Vertex item = null;
	    Iterator iter = cities.nodes.keySet().iterator();
	    while(iter.hasNext()) {
	      item = cities.nodes.get(iter.next());
	      
	      if (!item.equals(destvert)) {
		ListIterator tovert = item.adj.listIterator(0);
		while (tovert.hasNext()) // If it has an edge
		  {
		    Edge destEdge = (Edge) tovert.next();
		    Vertex direct = destEdge.getInfo().getDest();
		    if (direct.equals(destvert))
		      {
			available = true;
			System.out.println(destEdge.getInfo());
		      }
		  }
	      }
	    }
	    if (!available) System.out.println("There are no flights to " 
					       + destCity + ".");
	    break;
	    
	    //----------------------------------------------------------------
	    // Add a new flight
	    //----------------------------------------------------------------
	  case 'a':
	    System.out.println("Please enter departure city: ");
	    System.out.flush(); kbdin.nextToken(); 
	    String namefrom = kbdin.sval;
	    if (!airports.containsKey(namefrom)) {
	      System.out.println("Please enter valid city.");
	      continue;
	    }
	    System.out.println("Please enter destination city: ");
	    System.out.flush(); kbdin.nextToken(); 
	    String nameto = kbdin.sval;
	    if (!airports.containsKey(nameto)) {
	      System.out.println("Please enter valid city.");
	      continue;
	    }
	    System.out.println("Please enter flight name: ");
	    System.out.flush(); kbdin.nextToken(); 
	    String flightname = kbdin.sval;
	    System.out.println("Please enter flight number: ");
	    System.out.flush(); kbdin.nextToken(); 
	    int flightnum = (int) kbdin.nval;	    
	    System.out.println("Please enter departure time: ");
	    System.out.flush(); kbdin.nextToken(); 
	    int departime = (int) kbdin.nval;	    
	    System.out.println("Is that AM or PM? (Enter A or P): ");
	    System.out.flush(); kbdin.nextToken(); 
	    String dAmPm = kbdin.sval.toUpperCase();	    
	    System.out.println("Please enter arrival time, " 
			       + "local @ destination: ");
	    System.out.flush(); kbdin.nextToken(); 
	    int arrivaltime = (int) kbdin.nval;	    
	    System.out.println("Is that AM or PM? (Enter A or P): ");
	    System.out.flush(); kbdin.nextToken(); 
	    String aAmPm = kbdin.sval.toUpperCase();	    
	    
	    FlightInfo plane = new FlightInfo(flightname, flightnum, 0);
	    plane.addDepart(departime); plane.addDampm(dAmPm);
	    plane.addArrival(arrivaltime); plane.addAampm(aAmPm);
	    Vertex vfrom = null, vto = null;
	    vfrom = cities.nodes.get(namefrom); 
	    vto = cities.nodes.get(nameto); 
	    
	    manageAmPm(departime, dAmPm, arrivaltime, 
		       aAmPm, vfrom, vto,
		       plane, cities); 	
	    
	    break;

	    //----------------------------------------------------------------
	    // Unweighted Shortest Path
	    //----------------------------------------------------------------
	  case 'u':
	    System.out.println("Enter departing city: ");
	    System.out.flush(); kbdin.nextToken();
	    String startA = kbdin.sval;
	    if (!airports.containsKey(startA)) 
	      {
		System.out.println("Please enter valid city.");
		continue;
	      }
	    
	    System.out.println("Enter arriving city: ");
	    System.out.flush(); kbdin.nextToken();
	    String endB = kbdin.sval;
	    if (!airports.containsKey(endB)) 
	      {
		System.out.println("Please enter valid city.");
		continue;
	      }

	     Stack<Vertex> stack = cities.unweighted( startA,endB );
	     cities.printPath ( endB, "u" );
	     break;
	     
	    //----------------------------------------------------------------
	    // Dijkstra's algorithm for shortest path
	    //----------------------------------------------------------------
	  case 's':
	    System.out.println("Enter departing city: ");
	    System.out.flush(); kbdin.nextToken();
	    String cityA = kbdin.sval;
	    if (!airports.containsKey(cityA)) {
	      System.out.println("Please enter valid city.");
	      continue;
	    }
	    System.out.println("Enter arriving city: ");
	    System.out.flush(); kbdin.nextToken();
	    String cityB = kbdin.sval;
	    if (!airports.containsKey(cityB)) {
	      System.out.println("Please enter valid city.");
	      continue;
	    }
	    cities.dijkstra( cityA );
	    cities.printPath ( cityB, "d" );
	    break;
	    
	  default: 
	    menu();
	    System.out.println("Please enter one of i, d, a, f, r, m or q");
	    break;
	  }
	
	System.out.print("Next command: "); System.out.flush();
	kbdin.nextToken(); response = kbdin.sval;
      }
  }

  /**
   * Static method to print the menu.
   */  
  private static void menu() 
  {
    System.out.println("Please choose one of the following:"); 
    System.out.println("i -- insert a new city");
    System.out.println("a -- add a new flight");
    System.out.println("d -- display direct flights");
    System.out.println("s -- display shortest itinerary");
    System.out.println("f -- display departing flights from city");
    System.out.println("r -- display flights arriving flights to city");  
    System.out.println("u -- display unweighted shortest path");
    System.out.println("m -- to see menu again");
    System.out.println("q -- quit");
  }

  /**
   * Static method to manage the time for each line found in 
   * the file. Calculated time differences using Timer function
   * and adds destination to the flight information, and adds
   * an edge to the graph of cities.
   * @param int departure time
   * @param String departure am or pm letter
   * @param int arrival time
   * @param String arrival am or pm letter   
   * @param Vertex departure airport
   * @param Vertex arrival airport
   */  
  private static void manageAmPm(int depar, String dam, int arri, 
				 String aam, Vertex vf, Vertex vt,
				 FlightInfo p, Graph cit) 
  {
    Timer timex = new Timer();
    int totaltime;
    totaltime = timex.difference(arri, aam, depar, dam, vf, vt);
  
    p.addDest(vt); // Add destination to plane
    Edge e1 = new Edge(p, totaltime); // Add new edge
    cit.addedge(vf, e1); // Add to graph
   } 
}


