import java.util.Stack;
import java.util.ListIterator;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.HashMap;
import java.util.PriorityQueue;
import java.util.Queue;
import java.util.LinkedList;
import java.util.Map;

//********************************************************************
//  Graph.java       Author: Rio (Ryo Akasaka) and Hahna Kane
//  
//  Last modified: Friday, May 11, 2007
//
//  Contains a HashMap of cities that relates vertices to each other.
//  Also includes Dijkstra's algorithm for calculating shortest path.
//
//********************************************************************

public class Graph {
  
  protected Map<String, Vertex> nodes;     // The Vertices of this graph
  protected Map<String, Connect> stored;    // Stored airports
  protected Stack<Vertex> multiple;
  Timer time = new Timer();
  int printcount1;
  int printcount2;
  int store;
  protected int total;
  
  /**
   * Constructor that creates a new HashMap with
   * a String key containing Vertices
   */
  public Graph() {
    nodes = new HashMap<String, Vertex>();
  }

  /**
   * Adds vertex to HashMap
   * @param Vertex of airport
   */  
  public void addvertex(Vertex vert) {
    nodes.put(((Airport) vert.getVertex()).getAcronym(), vert);
  }
  
  /**
   * Adds an edge, where fromvert and tovert already are vertices
   * @param from and to objects already in hashMap
   */  
  public void addedge(Object fromvert, Object tovert) {
    ((Vertex) fromvert).addEdgeTo( (Edge) tovert );
  }
  
  /**
   * Unweighted shortest path algorithm that considers each
   * flight to be of length 1.
   * @param String of start vertex and end vertex acronyms
   * @return Stack of multiple airports. (This was not implemented)
   */    
  public Stack<Vertex> unweighted (String startVert, String endvert)
  {
    reset();
    stored = new HashMap<String, Connect>();
    multiple = new Stack<Vertex>();
    Vertex origin=nodes.get(startVert);
    Vertex insert=null;
    String end=endvert;
    Queue<Vertex> q = new LinkedList<Vertex>();
    q.add(origin);
    origin.addDist(0);
    int timedif=0;

    while (!q.isEmpty())
      {
	Vertex v = q.remove();
	ListIterator iter = v.adj.listIterator(0);
	while(iter.hasNext()) 
	  {
	    Edge e = (Edge) iter.next();
	    Vertex w = ((FlightInfo) e.getInfo()).getDest();
	    
	    String destin = ((Airport) w.getVertex()).getAcronym();
	    
	    Connect reveal;
	    if ((reveal = stored.get(((Airport) v.getVertex()).getAcronym())) != null) 
	   
	      {
		// If the airport has already been analyzed as
		// an arrival city, find layover time.
		
		int arrives = reveal.getFlight().getArrival(); 
		String aam = reveal.getFlight().getAampm(); 
		
		int departs =  ((FlightInfo) e.getInfo()).getDepart();
		String dam = ((FlightInfo)e.getInfo()).getDampm(); 
		
		timedif = time.difference(arrives, aam, departs, dam);	
		
	      }
	    
	    if (w.getDist()==INFINITY
		 && ((FlightInfo) e.getInfo()).getNumStops() < 1 
		 )
	      {
		w.addDist(v.getDist()+1);
		w.addPrev(v);
		w.addFlight(e);
		w.addCost(e.getCost());
		q.add(w);
				
		Connect flight = new 
		  Connect(e.getInfo(), timedif); 
		// Store shortest flight and the airport from which it arrived
		stored.put(destin, flight); 
	      }
	  }
      }
    
    return multiple; 
    // From attempt to add multiple airports
    // Demonstration purposes only
  }
  
  /**
   * Dijskstra's shortest path algorithm, where each flight
   * has a cost (the flight's time).
   * @param String of start vertex
   */     
  public void dijkstra(String startVert)
  {
    PriorityQueue<Path> pq= new PriorityQueue<Path>();
    PriorityQueue<Integer> layover = new PriorityQueue<Integer>();
    Vertex origin = nodes.get(startVert);
    
    reset();
    stored = new HashMap<String, Connect>();
    pq.add(new Path(origin, 0));
    origin.addCost(0);
    int departs = 0;
    int arrives = 0;
    int nodesChecked = 0;
    int totaltime = 0;
   
    String aam = "", dam ="";
    int counter = 0;
    while (!pq.isEmpty() && nodesChecked < nodes.size())
      {
	Path curr = pq.remove();
	Vertex x = curr.dest;
	
	if (x.getScratch() != 0)
	  continue;
	x.addScratch(1);
	nodesChecked++;
	
	ListIterator iter = x.adj.listIterator(0);
	while(iter.hasNext()) {
	  Edge e = (Edge) iter.next();
	  Vertex w = ((FlightInfo) e.getInfo()).getDest();
	  String destin = ((Airport) w.getVertex()).getAcronym();
	  
	  Connect reveal;
	 	  
	  if ((reveal = 
	       stored.get(((Airport) x.getVertex()).getAcronym())) != null) 
	    {
	      	      
	      arrives = reveal.getFlight().getArrival(); 
	      aam = reveal.getFlight().getAampm(); 
	      
	      departs =  ((FlightInfo) e.getInfo()).getDepart(); 
	      dam = ((FlightInfo) e.getInfo()).getDampm(); 
	      
	      totaltime = time.difference(arrives, aam, departs, dam);
	      
	      layover.add(totaltime);
	    }
	  
	  int cost = e.getCost();
	  if (cost < 0) {
	    System.out.println("Negative edges!");
	    System.out.println(((FlightInfo) e.getInfo()));
	    System.out.println(cost);
	    break;
	  }
	  if (w.getCost() > x.getCost() + cost 
	      && ((FlightInfo) e.getInfo()).getNumStops() == 0)
	    {
	      int compare;
	      if ( layover.peek() != null 
		   && totaltime > layover.peek()) {
		// Attempt to find least longest layover time
		// Demonstration purposes only
	      }
	      w.addCost(x.getCost() + cost);
	      w.addPrev(x);
	      w.addFlight(e);
	      pq.add(new Path(w, w.getCost()));
	      
	      Connect flight = 
		new Connect(e.getInfo(), totaltime); 
	      
	      stored.put(destin, flight); 
	    }
	}
      }
  }
  
  /**
   * Print path, String input.
   * Prints the total flight time and uses printPath vertex input
   * to perform recursive call on the edges of the previous flight
   * @param String of destination acronym
   * @param String dijkstra or unweighted
   */   
  public void printPath( String destName, String d )
  {
    Vertex origin = nodes.get(destName);
    if( origin == null )
      throw new NoSuchElementException( "Destination vertex not found" );
    else if( origin.getCost() == INFINITY )
      System.out.println("No flight itinerary could be made to: " + destName);
    else
      {
	System.out.println( );
	if (d.equals("d"))
	time.printTotal(origin.getCost(), 0);
	printPath( origin , d );
	System.out.println( );
      }
  }

  /**
   * Print path, Vertex inout.
   * Performs recursive call on the edges of the previous flight.
   * @param Vertex of destination
   * @param String dijkstra or unweighted
   */     
  private void printPath( Vertex dest, String d )
  {
    
    if( dest.getPrev() != null)
      {
	Stack<Edge> pile = new Stack<Edge>();
	pile.push(dest.getFlight());
	int totalflightime = dest.getCost();

	int layover = 0;
	Connect flight = stored.get(((Airport) dest.getVertex()).getAcronym());
	layover = flight.getTime();
	if (layover >= 30) {
	System.out.println("You have sufficient time to connect.");
	time.printTotal(layover, 1); 
	}
	else if (layover > 0 && layover < 30) {
	System.out.println("You do not have sufficient time to connect.");
	time.printTotal(layover, 1); 
	}
	if (printcount1 == 0 && d.equals("d")) {
	time.printTotal(layover + totalflightime, 3);
	System.out.println();
	printcount1 = 1;
	}
	printPath( dest.getPrev() , d);
	
	System.out.print( " to " );
	Edge print = pile.pop();
	System.out.println(print.getInfo());
	
	time.printTotal(print.cost, 2);
	store += print.cost;
	time.printTotal(store, 0);

      }
    System.out.print( ((Airport) dest.getVertex()).getAcronym() );
    
	
  }

  /**
   * Reset function that clears all values in the hashMap
   * for all previous edges and flights. Sets cost and distance
   * to infinity.
   */  
  public void reset() {
    Iterator iter = nodes.keySet().iterator();
    while(iter.hasNext()) {
      Vertex item = nodes.get(iter.next());
      item.addCost(INFINITY);
      item.addPrev(null);
      item.addDist(INFINITY);
      item.addScratch(0);
      store = 0;
      printcount1 = 0;
      printcount2 = 0;
    }
  }
  
  int INFINITY = 10000000;
}
