import java.util.StringTokenizer;

//********************************************************************
//  Timer.java       Author: Rio (Ryo Akasaka) and Hahna Kane
//
//  Last modified: Friday, May 11, 2007
//
//  Deals with GMT time differences, departure and arrival time
//
//********************************************************************
  
public class Timer {
    
  protected Integer cost;
  public Timer() {
  }
  
  /**
   * Converts hour time to minutes for comparing
   * @param int hour + minutes concatenated
   */    
  public int makeMinutes(int c) {
    cost = c;
    Integer hour = 0;
    Integer mins = 0;
    if (c < 0) {
      System.out.println("Negative error" + c);
    }
    // The flight only takes X minutes
    if (cost.toString().length() == 1)
      { hour = 0;
	mins = Integer.parseInt(cost.toString().substring(0, 1));}
    if (cost.toString().length() == 2) 
      { hour = 0;
	mins = Integer.parseInt(cost.toString().substring(0, 2));}
    // The flight has an hour component
    if (cost.toString().length() == 3) 
      { hour = Integer.parseInt(cost.toString().substring(0, 1));
	mins = Integer.parseInt(cost.toString().substring(1, 3));}
    // The flight has a two digit hour component as well as
    // a minutes component
    else if (cost.toString().length() == 4) 
      { hour = Integer.parseInt(cost.toString().substring(0, 2)); 
	mins = Integer.parseInt(cost.toString().substring(2,4));}
    
    if (mins >= 60) {
      mins = mins % 60;
      hour += 1;
    }
    return ((hour * 60) + mins); // Returns number of minutes
  }

  /**
   * Prints total time in air, spent in layover, of flight, or
   * door to door based on provided choice. 
   * @param int minutes of time
   */  
  public void printTotal(int c, int choice) {
    
    if (c > 0) {
      int hour = c / 60;
      int mins = c % 60;
      if (choice == 0) 
	System.out.println("Total flight time: " + hour + " hours and "
			   + mins + " minutes." ); 
      else if (choice == 1)
	System.out.println("Layover: " + hour + " hours and "
			   + mins + " minutes." ); 
      else if (choice == 2)
	System.out.println("Flight duration: " + hour + " hours and "
			   + mins + " minutes." ); 
      else if (choice == 3)
	System.out.println("Door to door time: " + hour + " hours and "
			   + mins + " minutes." ); 
    }
    else {
      System.out.println("Negative flight time. Check for errors.");
    }
  }

 
  /**
   * Arrival - departure difference calculator for flight information. 
   * Calculates the difference by using the GMT offset of the departure
   * and arrival cities, and redefines the arrival time based on the
   * timezone of the departure city. Then it calculates the time of the
   * flight based on the difference between the two, with several
   * special restrictions on morning/afternoon combinations.
   * @param int arrival time
   * @param String arrival am or pm
   * @param int departure time
   * @param String departure am or pm   
   * @param Vertex corresponding to arrival city
   * @param Vertex corresponding to departure city 
   */   
   public int difference(int arrives, String aam, 
			int departs, String dam, 
			Vertex vfrom, Vertex vto) {
    
    // Given two times in 2400 hour, find the difference between the two
    Timer time = new Timer();
    
    
    if (dam.equals("P")) {
      if (!(((Integer) departs).toString().substring(0, 2).equals("12") 
	    && ((Integer) departs).toString().length() == 4))
	departs += 1200;
    }
    
    // If departure is at midnight hour, make it 2400
    if (dam.equals("A") 
	&& (((Integer) departs).toString().substring(0, 2).equals("12") 
	    && ((Integer) departs).toString().length() == 4))
      departs -= 1200;
    
    if (aam.equals("P")) {
      if (!(((Integer) arrives).toString().substring(0, 2).equals("12")
	    && ((Integer) arrives).toString().length() == 4))
	arrives += 1200;
    }
    
    if (aam.equals("A") 
	&& (((Integer) arrives).toString().substring(0, 2).equals("12")
	    && ((Integer) arrives).toString().length() == 4))
      arrives -= 1200;
	      
    // Returns departure city offset GMT
    int depGMT = ((Airport) vto.getVertex()).getGMT(); 
    
    // Returns arrival city offset GMT
    int arrGMT = ((Airport) vfrom.getVertex()).getGMT();
    
    arrives = (arrives +  (arrGMT - depGMT)*100);
    
    // If after GMT analysis the arrival time is negative
    // it means it arrives in the late evening.
    if (arrives < 0) {
      arrives = 2400 + arrives;
      aam = "P";
    }
 
    arrives = time.makeMinutes(arrives);
    departs = time.makeMinutes(departs);
    int totaltime = (arrives - departs);
    

    
    // If the flight arrives in the evening and departs in the morning
    if (dam.equals("P") && aam.equals("A")) {
      totaltime = time.makeMinutes(2400) - (departs - arrives);
    }
    return totaltime;
  }
  

  /**
   * Arrival - departure difference calculator for layover information. 
   * Does not require GMT offset to calculate.
   * @param int arrival time
   * @param String arrival am or pm
   * @param int departure time
   * @param String departure am or pm   
   */   
  public int difference(int departs, String dam, int arrives, String aam) {
    
    // Given two times in 2400 hour, find the difference between the two
    Timer time = new Timer();
    
    // If departure is in afternoon, but not midday, add 1200
    if (dam.equals("P")) {
      if (!(((Integer) departs).toString().substring(0, 2).equals("12") 
	    && ((Integer) departs).toString().length() == 4))
	departs += 1200;
    }
    
    // If departure is at midnight hour, make it 0000
    if (dam.equals("A") 
	&& (((Integer) departs).toString().substring(0, 2).equals("12") 
	    && ((Integer) departs).toString().length() == 4))
      departs -= 1200;
    
    if (aam.equals("P")) {
      if (!(((Integer) arrives).toString().substring(0, 2).equals("12")
	    && ((Integer) arrives).toString().length() == 4))
	arrives += 1200;
    }
    
    if (aam.equals("A") 
	&& (((Integer) arrives).toString().substring(0, 2).equals("12")
	    && ((Integer) arrives).toString().length() == 4))
      arrives -= 1200;  
    
    if (arrives < 0) {
      arrives = 2400 + arrives;
      aam = "P";
    }
    
  
    arrives = time.makeMinutes(arrives);
    departs = time.makeMinutes(departs);
    int totaltime = (arrives - departs);

if (totaltime < 0)
       totaltime = time.makeMinutes(2400) + (arrives - departs);
 
         
    return totaltime;
    
  }

  /**
   * Manages time by splicing file input into hour time and
   * integer with am or pm letter
   * @param StringTokenizer string consisting of both int and String
   * @param int arrival or departure determiner
   * @param FlightInformation plane
   */   
  public void manageTime(StringTokenizer st, 
			 int arrdep, FlightInfo plane) 
  {
    String time;
    String ampm;
    int departime;
    time = st.nextToken();
    if (time.length() > 4) { 
      departime = Integer.parseInt(time.substring(0,4));
      ampm = time.substring(4,5);
    }
    else {
      departime = Integer.parseInt(time.substring(0,3));
      ampm = time.substring(3,4);
    }
    if (arrdep == 0) { plane.addDepart(departime); plane.addDampm(ampm); }
    else { plane.addArrival(departime); plane.addAampm(ampm); }
  }
}