import java.util.LinkedList;
import java.util.List;

//********************************************************************
//  Vertex.java       Author: Rio (Ryo Akasaka) and Hahna Kane
//
//  Last modified: Friday, May 11, 2007
//
//  Represents the vertices of the graph. Contains information about
//  each airport.
//
//********************************************************************

public class Vertex {
  protected List color;       
  protected LinkedList<Edge> adj;
  protected Object name;
  protected int cost;
  protected int scratch;
  protected Vertex prev;
  protected int dist;
  protected Edge prevfl;
  
  /**
   * Constructor. Creates a new list of adjacent
   * edges, empty name, empty cost, and scratch
   * variable to test if it has already been visted
   * or not
   */   
  public Vertex() {
    adj = new LinkedList<Edge>();
    name = " ";
    cost = 0;
    scratch = 0;
  }
  
  public Vertex(Object newvert) {
    adj = new LinkedList<Edge>();	
    name = newvert;	
    cost = 0;	
    scratch = 0;	
  }
  
  /**
   * Adds an edge
   * @param Edge object
   */   
  public void addEdgeTo(Edge e) {
    adj.addFirst(e);
  }

  /**
   * Comparator. Compares the acronym names of two airport
   * objects
   */   
  public boolean equals(Object other) {
    if (other instanceof Vertex)
      return (((Airport) (this.name)).getAcronym()
	      .equals(((Airport) ((Vertex) other).getVertex()).getAcronym()));
    else return false;
  }
  
  /**
   * printEdge method to print edge for stored airport object
   */   
  public String printEdge() {
    return ("\nDestination: "+ ((Airport) name).longName());
  }
  
  public String toString() {
    return ("\nDeparts: "+ ((Airport) name).longName());
  }
  
  /**
   * Adds flight that connects to this airport
   * @param Edge
   */   
  public void addFlight(Edge e) {
    prevfl = e;
 }
  
  public Edge getFlight() {
    return prevfl;
  }
  public Object getVertex()
  {
    return name;
  }

  public int getCost()
  {
    return cost;
  }
  
  public void addCost(int c)
  {
    cost = c;
  }
  public int getDist()
  {
    return dist;
  }
  
  public void addDist(int c)
  {
    dist = c;
  }
  
  public int getScratch()
  {
    return scratch;
  }
  
  public void addScratch(int s)
  {
    scratch = s;
  }
  
  public void addPrev(Vertex v)
  {
    prev = v;
  }
  
  public Vertex getPrev()
  {
    return prev;
  }
  }
